Imports System.IO
Imports System.Text

'---------------------------------------------------------
'LEDDisplay control written by John 'RuffNekk' Willemse
'Free to use and modify
'---------------------------------------------------------

'The LEDDisplay class is a user control acting as a LED display
'It has various methods to draw text and lines on it in various levels of intensity
Public Class LEDDisplay
  Inherits System.Windows.Forms.UserControl

#Region " Windows Form Designer generated code "
  Public Sub New()
    MyBase.New()

    'This call is required by the Windows Form Designer.
    InitializeComponent()

    'Add any initialization after the InitializeComponent() call
    SetStyle(ControlStyles.AllPaintingInWmPaint Or ControlStyles.DoubleBuffer Or ControlStyles.UserPaint, True)
    InitializeMatrices()
  End Sub

  'UserControl overrides dispose to clean up the component list.
  Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
    If disposing Then
      If Not (components Is Nothing) Then
        components.Dispose()
      End If
    End If
    MyBase.Dispose(disposing)
  End Sub

  'Required by the Windows Form Designer
  Private components As System.ComponentModel.IContainer

  'NOTE: The following procedure is required by the Windows Form Designer
  'It can be modified using the Windows Form Designer.  
  'Do not modify it using the code editor.
  <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
    '
    'LEDDisplay
    '
    Me.BackColor = System.Drawing.Color.Black
    Me.Font = New System.Drawing.Font("Verdana", 8.25!, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, CType(0, Byte))
    Me.ForeColor = System.Drawing.Color.Lime
    Me.Name = "LEDDisplay"
    Me.Size = New System.Drawing.Size(600, 380)

  End Sub

#End Region

  'intDots is an array that stores the state of each dot on the display
  'a value of 0 is OFF, 1 to 3 are increasing levels of brightness
  Private intDots(199, 126) As Integer
  'cDotColors is an array that stores the color for each level of brightness
  Private cDotColors() As Color = New Color() {Color.FromArgb(255, 0, 70, 0), Color.DarkGreen, Color.ForestGreen, Color.Lime}
  'The alCharacters arraylist stores the matrices for letters read from the file Matrices.txt
  Private alCharacters As New ArrayList

  'Overridden method OnPaint to handle all painting
  'The method is automatically called when the control is (partially) invalidated
  Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
    Dim G As Graphics = e.Graphics

    G.FillRectangle(Brushes.Black, e.ClipRectangle)

    'Draw each dot on the invalidated part of the display using the corresponding intensity
    For x As Integer = 0 To 199
      For y As Integer = 0 To 126
        If e.ClipRectangle.Contains(x * 3, y * 3) Then
          G.FillEllipse(New SolidBrush(cDotColors(intDots(x, y))), x * 3, y * 3, 2, 2)
        End If
      Next
    Next
  End Sub

  'The Clear method clears the entire display by setting all dots to zero
  Friend Sub Clear()
    For x As Integer = 0 To 199
      For y As Integer = 0 To 126
        intDots(x, y) = 0
      Next
    Next
    'Force a redraw of the control
    Me.Invalidate()
  End Sub

  'The DrawLetter method draws a letter on the screen at the dot specified.
  'x: the x coordinate
  'y: the y coordinate
  'letter: the letter to draw
  'intensity: the intensity to draw the letter in (default 3)
  Friend Sub DrawLetter(ByVal x As Integer, ByVal y As Integer, ByVal letter As Char, Optional ByVal intensity As Integer = 3)
    Dim sMatrix As String() = GetMatrix(letter) 'Get the matrix for the letter

    'Draw the matrix on the screen
    'An underscore (_) means the dot is OFF
    'An X means the dot is ON
    For row As Integer = 0 To 4
      For col As Integer = 0 To 4
        If sMatrix(row).Chars(col) = "_"c Then intDots(x + col, y + row) = 0
        If sMatrix(row).Chars(col) = "X"c Then intDots(x + col, y + row) = intensity
      Next
    Next

    'Force redraw
    Me.Invalidate(New Rectangle(x * 3, y * 3, 20, 20))
  End Sub

  'The DrawWord method draws a word on the screen at the specified coordinate
  'The method calls DrawLetter for each letter in the word
  Friend Sub DrawWord(ByVal x As Integer, ByVal y As Integer, ByVal word As String, Optional ByVal intensity As Integer = 3)
    For Each letter As Char In word.ToCharArray
      DrawLetter(x, y, letter, intensity) 'Draw the current letter
      x += 5 'Move to the right for the next letter
    Next
  End Sub

  'The DrawHorizontalLine method draws a horizontal line on the display
  Friend Sub DrawHorizontalLine(ByVal x1 As Integer, ByVal x2 As Integer, ByVal y As Integer, Optional ByVal intensity As Integer = 3)
    'Check if the line is from left to right or right to left
    If x2 >= x1 Then
      For x As Integer = x1 To x2 'Set all points from x1 to x2 on y to the value if intensity
        intDots(x, y) = intensity
      Next
      'Force redraw
      Me.Invalidate(New Rectangle(x1 * 3, y * 3, (x2 - x1) * 3, 2))
    Else
      For x As Integer = x2 To x1 'Set all points from x2 to x1 on y to the value if intensity
        intDots(x, y) = intensity
      Next
      Me.Invalidate(New Rectangle(x2 * 3, y * 3, (x1 - x2) * 3, 2))
    End If
  End Sub

  'The DrawVerticalLine method draws a vertical line on the display
  Friend Sub DrawVerticalLine(ByVal x As Integer, ByVal y1 As Integer, ByVal y2 As Integer, Optional ByVal intensity As Integer = 3)
    'Check if the line is from top to bottom or vice versa
    If y2 >= y1 Then
      For y As Integer = y1 To y2
        intDots(x, y) = intensity
      Next
      Me.Invalidate(New Rectangle(x * 3 - 1, y1 * 3, (y2 - y1) * 3, 2))
    Else
      For y As Integer = y2 To y1
        intDots(x, y) = intensity
      Next
      Me.Invalidate(New Rectangle(x * 3 - 1, y2 * 3, (y1 - y2) * 3, 2))
    End If
  End Sub

  'The DrawRectangle method draws a rectangle on the display by drawing two horizontal and two vertical lines
  Friend Sub DrawRectangle(ByVal x As Integer, ByVal y As Integer, ByVal width As Integer, ByVal height As Integer, Optional ByVal intensity As Integer = 3)
    DrawHorizontalLine(x, x + width, y, intensity) 'Top
    DrawHorizontalLine(x, x + width, y + height, intensity) 'Bottom
    DrawVerticalLine(x, y, y + height, intensity) 'Left
    DrawVerticalLine(x + width, y, y + height, intensity) 'Right
  End Sub

  'The Dot property gets or sets the intensity value of any dot on the display
  Friend Property Dot(ByVal x As Integer, ByVal y As Integer) As Integer
    Get
      Return intDots(x, y)
    End Get
    Set(ByVal Value As Integer)
      intDots(x, y) = Value
      'Force redraw to show the new value
      Me.Invalidate()
    End Set
  End Property

  'The GetMatrix function return the matrix of a given letter.
  'The matrix is a 5 x 5 block of characters where an _ means the dot must be turned off and an X means to turn it on
  Private Function GetMatrix(ByVal letter As Char) As String()
    For Each ch As Character In alCharacters 'Loop through all letters
      If ch.Char = letter Then Return ch.Matrix 'Return the matrix if the letter matches
    Next

    Return Nothing 'Return nothing if the letter was not found
  End Function

  'The InitializeMatrices method loads the matrices from the file Matrices.txt (embedded resource) and
  'stores them in Character structures and finally in an arraylist alCharacters
  Private Sub InitializeMatrices()
    Dim sReader As StreamReader
    Dim sLetter As String
    Dim sReturn(4) As String
    Dim cTemp As Character

    Try
      'Open the file for reading
      sReader = New StreamReader(System.Reflection.Assembly.GetExecutingAssembly.GetManifestResourceStream("Hangman.Matrices.txt"))
      sLetter = CStr(sReader.ReadLine) 'Get the first line of the file
      While Not sLetter Is Nothing 'While the read line contains something
        If sLetter.EndsWith(":") Then 'If the line ends with a colon, then the next 5 lines are a matrix
          sReturn = New String() {"", "", "", "", ""} 'Empty the array to prepare it for new content
          For i As Integer = 0 To 4 'Get the next 5 lines
            sReturn(i) = sReader.ReadLine 'Store them in the array
          Next
          cTemp = New Character(sLetter.Chars(0), sReturn) 'Create a new Character structure to store it
          alCharacters.Add(cTemp) 'Add the structure to the arraylist for later use
        End If
        sLetter = CStr(sReader.ReadLine) 'Get the next line of text
      End While
    Catch ex As Exception 'Unhandled as of yet
    Finally
      If Not sReader Is Nothing Then sReader.Close() 'Close the reader
    End Try
  End Sub

  'The Character structure holds information about a character and its matrix
  Private Structure Character
    Private _Char As Char         'The character
    Private _Matrix() As String   'The matrix

    Friend Sub New(ByVal letter As Char, ByVal matrix As String())
      _Char = letter
      _Matrix = matrix
    End Sub

    Friend ReadOnly Property [Char]() As Char
      Get
        Return _Char
      End Get
    End Property

    Friend ReadOnly Property Matrix() As String()
      Get
        Return _Matrix
      End Get
    End Property
  End Structure
End Class
